CON
  _clkmode = xtal1 + pll16x     'Use the PLL to multiple the external clock by 16
  _xinfreq = 5_000_000          'An external clock of 5MHz. is used (80MHz. operation)


CON
  CR            = 13
  LF            = 10
  NBSP          = 32
  HTTP_ERROR    = 0
  HTTP_GET      = 1
  HTTP_POST     = 2
  HTTP_10       = 0
  HTTP_11       = 1
  DYNAMIC       = 3
  FS            = "/"
  EQ            = "="
  AMP           = "&"
  QM            = "?"
  DOT           = "."
  STR_TERM      = 0

OBJ

DAT
  get                   byte    "GET",  0
  post                  byte    "POST", 0
  http                  byte    "HTTP/1.1", 0
  host                  byte    "Host:", 0
  contlen               byte    "Content-Length:", 0
  body                  byte    CR, LF, CR, LF, 0
  dynamicpage           byte    "SHP", 0
  defaultpage           byte    "index.htm", 0, "default.htm", 0
  placeholder           byte    "<#0001#>", 0
  filestack             byte    $0[20], 0
  hostname              byte    $0[20], 0    
  dirstack              byte    $0[256], 0
  url                   byte    $0[256], 0
  'fileBuffer            byte    $0[2048], 0
  'clipboard             byte    $0[1024], 0
  varstack              byte    $0[1024], 0
  'dynamicContent        byte    "<h1>Hello World</h1>", 0

  
VAR
  byte  method
  byte  httpVersion
  byte  fileType
  byte  directoryStackDepth
  byte  postback  

  long  ptrStart
  long  ptrEnd
  long  contentLength
  long  header
  long  directoryStackPtr[10]
  long  fileStackPtr[2]
  long  fileErrorHandle
  long  varstackPtr



PUB Initialize(data)
  header := data
  
  bytefill(@dirstack, 0, 256)
  bytefill(@filestack, 0, 20)
  bytefill(@varstack, 0, 1024)
  bytefill(@url, 0, 256)
  directoryStackPtr[0] :=  @dirstack
  varstackPtr := @varstack

  FillHTTPMethod
  FillHeaderValue(@host, @hostname)
  FillContentLength
  FillURL
  EnumerateUrl
  FillFileType


PUB GetDirectoryStackDepth | i

  i := PeekDir(directoryStackDepth - 1)
  if(strsize(i) > 0)
    result := directoryStackDepth - 1
  else
    result := -1


PUB Filename | i
  ' Get the file name
  i := PeekDir(directoryStackDepth-1)
  if(strsize(i) > 0)
    result := i
  else
    result := @defaultpage

    
PUB Directory(index)
  result := PeekDir(index)

PUB GetFileType
  result := fileType
  
PUB FilenameNoExtension
  result := @filestack

PUB FileExtension
  result := filestackptr[1] 
  
PUB GetUrlPtr
  result := @url

PUB GetMethod
  result := method


PUB FillHTTPMethod
  
  if(MatchPattern(header, @get, false, 0) > -1)
    method := HTTP_GET
    return

  if(MatchPattern(header, @post, false, 0) > -1)
    method := HTTP_POST
    return

  method := HTTP_ERROR



        
  
PUB __POST(name) | s, e, l, offset

  result := -1

  ' Make sure that we have a post
  if(method <> HTTP_POST)
    return

  '  check the content length
  if(contentLength == 0)
    return

  ' Position the pointer to the body section of the header
  offset := MatchPattern(header, @body, true, 0)
  if(offset == -1)
    return

  ' Move the offset past the search string
  offset := offset + strsize(@body)
   
  ' Find the name/value
  offset := MatchPattern(header, name, false, offset)
  if(offset == -1)
    return

  offset := offset + strsize(name)

  s := InString(header, EQ, false, offset)
  e := InString(header, AMP, false, s)


  ' Check for a CR
  if(e == -1)
    e := InString(header, STR_TERM, false, s) 

  if((s == e-1) or (s == e))
    return

  
  s := s + 1
  l := e - s

  s := s + header
  
  ' push variable on the varstack
  ' and return a pointer
  return PushVarStack(s, l)


PUB __GET(name) | s, e, offset, l
  'Parse the query string
  result := -1

  ' go to the start fo the querystring
  s := InString(@url, QM, true, 0)
  
  if(s == -1)
    return
  
  offset := MatchPattern(@url, name, true, s)
  if(offset == -1)
    return
  
  offset := offset + strsize(name)
   
  s := InString(@url, EQ, true, offset)
  e := InString(@url, AMP, true, s)
  
  if(e == -1)
    e :=  InString(@url, STR_TERM, true, s)

  ' If the character after the "=" is a "&"
  '  then we have an empty value
  if((s == e-1) or (s == e))
    return

  s := s + 1
  l := e - s
  s := s + @url
  
  return PushVarStack(s, l)


PRI PushVarStack(startAddress, bytesToWrite) | e, temp

  ' save the varstackPtr address
  temp := varstackPtr

  ' Write the variable to memory
  bytemove(varstackPtr, startAddress, bytesToWrite)

  ' Terminate the string with the old 0
  e := varstackPtr + bytesToWrite
  bytefill(e, STR_TERM, 1)

  ' Decode the string
  ' This might make the string a little smaller
  DecodeString(varstackPtr)
  
  'Update the pointer so it's pointing to the next address
  varstackPtr := strsize(varstackPtr) + 1

  ' Return a pointer to the string we just pushed on the stack
  return temp 


PRI DecodeString(decodeStr) | char, inPlace, outPlace
  inPlace := outPlace := 0
  repeat 
    char := byte[decodeStr][inPlace++]
    if (char == "%") ' convert %## back into a character
      'inPlace++ ' skip %
      ' first nibble
      char := byte[decodeStr][inPlace++] - 48
      if (char > 9)
        char -= 7
      char := char << 4
      byte[decodeStr][outPlace] := char
      ' second nibble
      char := byte[decodeStr][inPlace++] - 48
      if (char > 9)
        char -= 7
      byte[decodeStr][outPlace++] += char
      ' since we trashed char doing the decode, we need this to keep the loop going
      char := "x"
    elseif (char == "+") ' convert + back to a space
      byte[decodeStr][outPlace++] := " "
    else ' no conversion needed, just set the character
      byte[decodeStr][outPlace++] := char
  until (char == 0)
    
  byte[decodeStr][outPlace-1] := 0 ' terminate the string at it's new shorter size 
    
PUB GetFilePath | s, e

  ptrStart := 0
  ptrEnd := 0
  
  s := InString(@url, FS, true, 0)
  
  ' go to the start fo the querystring
  e := InString(@url, QM, true, 0)
  if( e == -1)
    e := InString(@url, STR_TERM, true, 0)

  ptrStart := s
  ptrEnd := e-1

  if(s == e)
    ptrStart := 0
    ptrEnd := 1
  

PUB FillFileType | s, e, i, temp, len

  fileType := 0   
  temp := Filename
  
  'fileStackPtr[0] := @filestack
  
  ' Get filename length
  len := STRSIZE(temp)

  ' Get the dot position
  ' We're expecting 8.3
  i := InString(temp, DOT, true, 0)

  'Fill the filestack
  bytemove(@filestack, temp, len)

  ' Replace the "." with a string terminator
  ' We're expecting 1 dot
  bytefill(@filestack+i, STR_TERM, 1)

  ' Get the position of the extension
  fileStackPtr[1] :=  @filestack+i+1


  'TODO - remove this as we always load a default file
  if(i == -1)
    fileType := 1
    ptrStart := 0
    ptrEnd := 1
    return

  if(MatchPattern(@url, @dynamicpage, false, 0) == -1)
    fileType := 2
  else
    fileType := 3

  return


PUB IsMethod(name)

  result := false

  if(MatchPattern(@filestack, name, false, 0) > -1)
    result := true
     

PUB EnumerateUrl | i, e, s, t

  directoryStackDepth := 0

  ' Find the end of the url, "?" or "0" 
  e := InString(@url, QM, true, 0)
  if( e == -1)
    e := InString(@url, STR_TERM, true, 0)

  ' Prime the pointers
  i := 0
  s := 1

  ' Get the first "/"  position
  ' Should be position 0
  i := InString(@url, FS, true, 0)

  repeat
    i := InString(@url, FS, true, s)
    if(i > -1)
      'PushDir directory name on to the stack 
      PushDir(@url+s, i-s)
      
      'Increment the start pointer so we're pointing
      'to the next directory name character not the "/" 
      s := i+1
      t := i
      
    if(i == -1)
      'PushDir the filename on the stack
      PushDir(@url+s, e-s)
      quit


PUB PushDir(startAddress, numberOfBytes) | e

  ' Pointer to the end of the string
  e := directoryStackPtr[directoryStackDepth] + numberOfBytes
  
  bytemove(directoryStackPtr[directoryStackDepth], startAddress, numberOfBytes)
  bytefill(e, STR_TERM, 1)
  directoryStackPtr[++directoryStackDepth]  :=  e+1
   


PUB PeekDir(index)

  if(index > directoryStackDepth-1)
    result := -1
    return
  result := directoryStackPtr[index]
  
  
       

PUB FillURL | i, j, x, char

  i := 0
  j := 0

  i := InString(header, FS, true, 0)
  j := InString(header, NBSP, true, i+1)

  ptrStart := i
  ptrEnd := j-1

  bytemove(@url, header+i, j-i )


  
PUB IsHTTPVersion11
  if(MatchPattern(header, @http, false, 0) > -1)
    result := true
  else
    result := false 


Pub FillContentLength | i, temp, s, e, l, offset, t1

  offset := MatchPattern(header, @contlen, false, 0)
  if(offset == -1)
    contentLength := 0
    return 0

  offset :=  offset + strsize(@contlen)
  
  s := offset + 1
  e := InString(header, CR, false, offset)
  l :=  e - offset - 1
  s := s + header
  
  if(e == -1)
    contentLength := 0
    return
   
  t1 := l-1
  contentLength := 0
   
  repeat i from 0 to l-1
    temp :=  PowerOfTen(t1--)    
    temp :=  (byte[s + i] - 48) * temp
    contentLength := contentLength + temp

    
PRI PowerOfTen(numOfTens) | i, temp

  temp := 1
  
  if(numOfTens > 0)
    repeat i from 1 to numOfTens
      temp := 10 * temp

  result := temp

 
PUB FillHeaderValue(valueToFind, fillAddress) | s, e, l, offset

  offset := MatchPattern(header, valueToFind, false, 0)
  offset := offset + strsize(valueToFind)

  e := InString(header, CR, false, offset)
  
  s := offset + 1
  l :=  e - offset - 1
  s := s + header

  bytemove(fillAddress, s,  l)
  bytefill(fillAddress + l, 0, 1)
  

PUB InString(source, char, matchCase, offset) | i, j, sourceLength, patternlength
  sourceLength := strsize(source)
  result := -1

  repeat i from offset to sourceLength
    if(IsCharMatch(source + i, @char, matchCase) )
      result := i
      quit   


PUB MatchPattern(source, pattern, matchCase, startIndex) | i, j, sourceLength, patternlength

  sourceLength := strsize(source)
  patternlength := strsize(pattern)
  result := -1
  ptrEnd := 0
  ptrStart := 0

  i := 0
  j := 0
   
  repeat i from startIndex to sourceLength
    ' Try to match the first char in pattern to the
    ' source 
    if(IsCharMatch(source + i, pattern + j, matchCase) )    
      'Found a match
      'Store the current pointer position and increment the source pointer by one byte

      ptrStart := source + i
      'Lets see if the pattern lines up with the source
      repeat j from 0 to patternLength - 1
        ifnot(IsCharMatch(source + i + j, pattern + j, matchCase))
          j := 0
          ptrEnd := -1
          ptrStart := -1
          quit
          
    if(j > 0)
      ptrEnd := ptrStart + patternLength
      result := i
      quit   

  
PRI IsCharMatch(sourceChar, patternChar, matchCase)
  '' Initilize the return value
  result := false


  if(matchCase)
    if(byte[sourceChar] == byte[patternChar])
      result := true   
  else
    if((byte[sourceChar] == byte[patternChar]) or (byte[sourceChar] == byte[patternChar]+32))
      result := true
  


'***************************************
PRI PauseMSec(Duration)
'***************************************
''  Pause execution for specified milliseconds.
''  This routine is based on the set clock frequency.
''  
''  params:  Duration = number of milliseconds to delay                                                                                               
''  return:  none
  
  waitcnt(((clkfreq / 1_000 * Duration - 3932) #> 381) + cnt)

  return  'end of PauseMSec


DAT
  Workspace BYTE 0